/*
 * Copyright 2020 Anton Trushkov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ksdfv.thelema.test.shaders.glsl

import org.ksdfv.thelema.g3d.Object3D
import org.ksdfv.thelema.g3d.Scene
import org.ksdfv.thelema.g3d.cam.ActiveCamera
import org.ksdfv.thelema.g3d.cam.Camera
import org.ksdfv.thelema.gl.GL
import org.ksdfv.thelema.gl.GL_COLOR_BUFFER_BIT
import org.ksdfv.thelema.gl.GL_DEPTH_BUFFER_BIT
import org.ksdfv.thelema.math.IVec3
import org.ksdfv.thelema.math.Vec3
import org.ksdfv.thelema.mesh.build.BoxMeshBuilder
import org.ksdfv.thelema.shader.Shader
import org.ksdfv.thelema.shader.glsl.CameraDataNode
import org.ksdfv.thelema.shader.glsl.OutputNode
import org.ksdfv.thelema.shader.glsl.VertexNode
import org.ksdfv.thelema.test.Test
import org.ksdfv.thelema.utils.LOG

/** @author zeganstyl */
class VertexNodeTest: Test {
    override val name: String
        get() = "Vertex Node"

    override fun testMain() {
        ActiveCamera.api = Camera().apply {
            lookAt(Vec3(5f, 5f, 5f), IVec3.Zero)
            near = 0.1f
            far = 100f
            update()
        }

        val scene = Scene()

        scene.objects.add(Object3D().apply {
            position.set(2f, 0f, 0f)
            updateTransform()

            meshes.add(BoxMeshBuilder().apply {
                positionName = "POSITION"
                material.shader = Shader().apply {
                    val vertexNode = addNode(VertexNode())
                    vertexNode.aPositionName = "POSITION"
                    val cameraDataNode = addNode(CameraDataNode(vertexNode.position))
                    addNode(OutputNode(cameraDataNode.clipSpacePosition, vertexNode.position))

                    build()
                    LOG.info(sourceCode())
                }
            }.build())
        })

        GL.isDepthTestEnabled = true

        GL.render {
            GL.glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT)

            scene.render()
        }
    }
}