/*
 * Copyright 2020 Anton Trushkov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ksdfv.thelema.fs

import org.ksdfv.thelema.fs.FS.api

/** File system abstraction layer. It is singleton and you can use it anywhere after initialization.
 * If you want multiple APIs, you may switch between them by setting [api].
 * By default used JVM file system implementation.
 * @author zeganstyl */
object FS: IFS {
    const val ReadAccess = 0
    const val WriteAccess = 1
    const val ListFilesAccess = 2

    lateinit var api: IFS

    override val externalStoragePath: String
        get() = api.externalStoragePath

    override val isExternalStorageAvailable: Boolean
        get() = api.isExternalStorageAvailable

    override val localStoragePath: String
        get() = api.localStoragePath

    override val isLocalStorageAvailable: Boolean
        get() = api.isLocalStorageAvailable

    override fun file(path: String, location: Int) = api.file(path, location)
    override fun classpath(path: String) = api.classpath(path)
    override fun internal(path: String) = api.internal(path)
    override fun external(path: String) = api.external(path)
    override fun absolute(path: String) = api.absolute(path)
    override fun local(path: String) = api.local(path)

}